
;**************************************************************************
;*        Manual Counter 00-99                                            *
;*        2x7-seg.LED / 2 buttons                                         *
;**************************************************************************
;**************************************************************************
;*        v 1.10 - 22.12.2019                                             *
;**************************************************************************
;*        Hardw.: PIC16F1828 used                                         *
;*        OSC.......: Internal osc. used     POWER.....:   5,0V extern.   *
;**************************************************************************

 INCLUDE "P16F1828.INC"

_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
;--------------------------------------------------------------------------
v_cnt_H		EQU	H'22'
v_cnt_L		EQU	H'23'
;--------------------------------------------------------------------------
;==========================================================================
		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;              ++  RESET :  main boot routine  ++
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_main	nop
		clrf	BSR
		clrf	INTCON			;disable interrupts

		banksel	OPTION_REG
		movlw	B'00111111'
		movwf	OPTION_REG		;see datasheet Page 176
						;bit 7 = 0 --> Week pull-ups are enabled
		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		banksel	TRISA
		movlw	B'11111000'
		movwf	TRISA

		movlw	B'00000000'		;7-segment dirvers
		movwf	TRISC

		movlw	B'00001111'
		movwf	TRISB

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON			;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		banksel	WPUA
		movlw	B'00111000'
		movwf	WPUA
		
		clrf	BSR			;select BANK 0
		goto	main_prg
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Program
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_prg	clrf	BSR

		movlw	H'FF'
		movwf	PORTC

		call	w_minus		
		call	dr2

		call	ee_read
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_loop	call	w_number
		call	dr2
		btfss	PORTA,D'003'		;+
		call	cnt_inc
		btfss	PORTA,D'004'		;-
		call	cnt_dec
		btfss	PORTA,D'005'		;CLR
		call	cnt_clr
		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt_clr		call	w_space
		call	dr2
		call	w_number
		call	dr2

		call	w_space
		call	dr2
		call	w_number
		call	dr2

		call	w_space
		call	dr2
		call	w_number
		call	dr2

		btfsc	PORTA,D'005'
		return

		clrf	v_cnt_L
		clrf	v_cnt_H

cnt_clr_fin	call	dr2
		call	w_number
		btfss	PORTA,D'005'
		goto	cnt_clr_fin

		call	ee_write
		call	w_space
		call	dr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt_inc		call	dr2
		incf	v_cnt_L,1
		movf	v_cnt_L,0
		xorlw	D'010'
		btfss	STATUS,Z
		goto	cnt_inc_fin
		clrf	v_cnt_L
		incf	v_cnt_H,1
		movf	v_cnt_H,0
		xorlw	D'010'
		btfss	STATUS,Z
		goto	cnt_inc_fin
		clrf	v_cnt_H
;------------------------------------------------------------------------------
cnt_inc_fin	call	dr2
		call	w_number
		btfss	PORTA,D'003'
		goto	cnt_inc_fin

		call	ee_write
		call	w_space
		call	dr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt_dec		call	dr2
		decf	v_cnt_L,1
		movf	v_cnt_L,0
		xorlw	D'255'
		btfss	STATUS,Z
		goto	cnt_dec_fin
		movlw	D'009'
		movwf	v_cnt_L
		decf	v_cnt_H,1
		movf	v_cnt_H,0
		xorlw	D'255'
		btfss	STATUS,Z
		goto	cnt_inc_fin
		movlw	D'009'
		movwf	v_cnt_H
;------------------------------------------------------------------------------
cnt_dec_fin	call	dr2
		call	w_number
		btfss	PORTA,D'004'
		goto	cnt_dec_fin

		call	ee_write
		call	w_space
		call	dr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_number	movf	v_cnt_L,0
		call	w_dig_L

		movf	v_cnt_H,0
		call	w_dig_H
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_dig_H		brw
		goto	w_0x
		goto	w_1x
		goto	w_2x
		goto	w_3x
		goto	w_4x
		goto	w_5x
		goto	w_6x
		goto	w_7x
		goto	w_8x
		goto	w_9x
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_dig_L		brw
		goto	w_x0
		goto	w_x1
		goto	w_x2
		goto	w_x3
		goto	w_x4
		goto	w_x5
		goto	w_x6
		goto	w_x7
		goto	w_x8
		goto	w_x9
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
test_main_loop	nop

		call	w_0x
		call	w_x0
		call	dr2

		call	w_1x
		call	w_x1
		call	dr2

		call	w_2x
		call	w_x2
		call	dr2

		call	w_3x
		call	w_x3
		call	dr2

		call	w_4x
		call	w_x4
		call	dr2

		call	w_5x
		call	w_x5
		call	dr2

		call	w_6x
		call	w_x6
		call	dr2

		call	w_7x
		call	w_x7
		call	dr2

		call	w_8x
		call	w_x8
		call	dr2

		call	w_9x
		call	w_x9
		call	dr2

		goto	test_main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_minus		nop
		banksel	LATC
		bcf	LATC,D'000'
		bcf	LATC,D'001'
		bcf	LATC,D'002'
		bcf	LATC,D'003'
		bcf	LATC,D'004'
		bcf	LATC,D'005'
		bsf	LATC,D'006'

		bcf	LATA,D'000'
		bcf	LATA,D'001'
		bcf	LATA,D'002'
		bcf	LATB,D'004'
		bcf	LATB,D'005'
		bcf	LATB,D'006'
		bsf	LATB,D'007'

		clrf	BSR
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_space		nop
		banksel	LATC
		bcf	LATC,D'000'
		bcf	LATC,D'001'
		bcf	LATC,D'002'
		bcf	LATC,D'003'
		bcf	LATC,D'004'
		bcf	LATC,D'005'
		bcf	LATC,D'006'

		bcf	LATA,D'000'
		bcf	LATA,D'001'
		bcf	LATA,D'002'
		bcf	LATB,D'004'
		bcf	LATB,D'005'
		bcf	LATB,D'006'
		bcf	LATB,D'007'

		clrf	BSR
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_x0		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bsf	LATB,D'005'
		bsf	LATB,D'006'
		bcf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x1		nop
		banksel	LATC
		bcf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATB,D'004'

		bcf	LATB,D'005'
		bcf	LATB,D'006'
		bcf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x2		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bcf	LATA,D'002'
		bsf	LATB,D'004'

		bsf	LATB,D'005'
		bcf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x3		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bcf	LATB,D'005'
		bcf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x4		nop
		banksel	LATC
		bcf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATB,D'004'

		bcf	LATB,D'005'
		bsf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x5		nop
		banksel	LATC
		bsf	LATA,D'000'
		bcf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bcf	LATB,D'005'
		bsf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x6		nop
		banksel	LATC
		bsf	LATA,D'000'
		bcf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bsf	LATB,D'005'
		bsf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x7		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bcf	LATB,D'004'

		bcf	LATB,D'005'
		bcf	LATB,D'006'
		bcf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x8		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bsf	LATB,D'005'
		bsf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_x9		nop
		banksel	LATC
		bsf	LATA,D'000'
		bsf	LATA,D'001'
		bsf	LATA,D'002'
		bsf	LATB,D'004'

		bcf	LATB,D'005'
		bsf	LATB,D'006'
		bsf	LATB,D'007'
		clrf	BSR
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_0x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'
		bsf	LATC,D'004'
		bsf	LATC,D'005'
		bcf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_1x		nop
		banksel	LATC
		bcf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bcf	LATC,D'003'

		bcf	LATC,D'004'
		bcf	LATC,D'005'
		bcf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_2x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bcf	LATC,D'002'
		bsf	LATC,D'003'

		bsf	LATC,D'004'
		bcf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_3x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'

		bcf	LATC,D'004'
		bcf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_4x		nop
		banksel	LATC
		bcf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bcf	LATC,D'003'

		bcf	LATC,D'004'
		bsf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_5x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bcf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'

		bcf	LATC,D'004'
		bsf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_6x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bcf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'

		bsf	LATC,D'004'
		bsf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_7x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bcf	LATC,D'003'

		bcf	LATC,D'004'
		bcf	LATC,D'005'
		bcf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_8x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'

		bsf	LATC,D'004'
		bsf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------
w_9x		nop
		banksel	LATC
		bsf	LATC,D'000'
		bsf	LATC,D'001'
		bsf	LATC,D'002'
		bsf	LATC,D'003'

		bcf	LATC,D'004'
		bsf	LATC,D'005'
		bsf	LATC,D'006'
		clrf	BSR
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; EEPROM
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
ee_read		nop
		banksel	EEADRL
		movlw	H'00'			;EEPROM Address 0 -> v_cnt_H
		movwf	EEADRL

		bcf	EECON1, CFGS
		bcf	EECON1, EEPGD
		bsf	EECON1, RD
		movf	EEDATL, 0

		clrf	BSR
		movwf	v_cnt_H
;-----------------------------------------------------------
		banksel	EEADRL
		movlw	H'01'			;EEPROM Address 0 -> v_cnt_H
		movwf	EEADRL

		bcf	EECON1, CFGS
		bcf	EECON1, EEPGD
		bsf	EECON1, RD
		movf	EEDATL, 0

		clrf	BSR
		movwf	v_cnt_L
;-----------------------------------------------------------------------------
		movf	v_cnt_L,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return
;-----------------------------------------------------------------------------
;EERPOM is empty
		clrf	v_cnt_L
		clrf	v_cnt_H
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ee_write	nop
		banksel	EEADRL
		movlw	H'00'			;EEPROM Address 0 -> v_cnt_H
		movwf	EEADRL
		clrf	BSR
		movf	v_cnt_H,0
		call	ee_write_core

		nop
		banksel	EEADRL
		movlw	H'01'			;EEPROM Address 0 -> v_cnt_L
		movwf	EEADRL
		clrf	BSR
		movf	v_cnt_L,0
		call	ee_write_core
		return
;----------------------------------------------------------------------------
ee_write_core	nop
		banksel	EEADRL
		movwf	EEDATL

		bcf	EECON1, CFGS
		bcf	EECON1, EEPGD
		bsf	EECON1, WREN
		
		movlw	H'55'
		movwf	EECON2
		movlw	H'AA'
		movwf	EECON2

		bsf	EECON1, WR
		bcf	EECON1, WREN

wait_eew	btfsc	EECON1, WR
		goto	wait_eew

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
		END